﻿using AutoMapper;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading.Tasks;
using Tsi1.BusinessLayer.Dtos;
using Tsi1.BusinessLayer.Helpers;
using Tsi1.BusinessLayer.Interfaces;
using Tsi1.DataLayer;
using Tsi1.DataLayer.Entities;

namespace Tsi1.BusinessLayer.Services
{
    public class TenantService : ITenantService
    {
        private readonly Tsi1Context _context;

        private readonly IMapper _mapper;

        public TenantService(Tsi1Context context, IMapper mapper)
        {
            _context = context;
            _mapper = mapper;
        }

        public async Task<ServiceResult<List<TenantPreviewDto>>> GetAll()
        {
            var result = new ServiceResult<List<TenantPreviewDto>>();

            var tenants = await _context.Tenants.ToListAsync();

            var tenantsDto = _mapper.Map<List<TenantPreviewDto>>(tenants);

            result.Data = tenantsDto;

            return result;
        }

        public async Task<ServiceResult<int>> GetByName(string tenantName)
        {
            var result = new ServiceResult<int>();

            var tenant = await _context.Tenants
                .FirstOrDefaultAsync(x => x.Name == tenantName);

            if (tenant == null)
            {
                result.HasError = true;
                result.Message = string.Format(ErrorMessages.TenantDoesNotExist, tenantName);
                return result;
            }

            result.Data = tenant.Id;

            return result;
        }

        public async Task<ServiceResult<TenantPreviewDto>> Create(TenantCreateDto newTenant)
        {
            var result = new ServiceResult<TenantPreviewDto>();

            var existingTenant = await _context.Tenants.FirstOrDefaultAsync(x => x.Name == newTenant.Name);

            if (existingTenant != null)
            {
                result.HasError = true;
                result.Message = string.Format(ErrorMessages.DuplicateTenantName, newTenant.Name);
                return result;
            }

            var tenant = _mapper.Map<Tenant>(newTenant);
            _context.Tenants.Add(tenant);

            await _context.SaveChangesAsync();
         
            result.Data = _mapper.Map<TenantPreviewDto>(tenant);

            return result;
        }

    }
}
